using System;
using System.IO;
using System.Xml;
using System.Collections;
using System.Xml.Serialization;


public class DictionaryCollection<TValue> : DictionaryBase, IXmlSerializable
{
    private XmlSerializer valueSerializer;
    private XmlSerializer keySerializer;        
  
    public DictionaryCollection()
    {
        keySerializer = new XmlSerializer(typeof(String));
        valueSerializer = new XmlSerializer(typeof(TValue));
        
    }

    public TValue this[string key]
    {
        get
        {
            return (TValue)this.Dictionary[key];
        }
        set  
        { 
            this.Dictionary[key] = value; 
        }
    }

    public void Add(String key, TValue value)
    {
        this.Dictionary.Add(key, value);
    }


    public void WriteXml(System.Xml.XmlWriter writer)
    {
        foreach (String key in this.Dictionary.Keys)
        {
            keySerializer.Serialize(writer, key);
            valueSerializer.Serialize(writer, this[key]);
        }
    }

    public void ReadXml(System.Xml.XmlReader reader)
    {
        reader.Read();

        while (reader.NodeType != XmlNodeType.EndElement)
        {
            String key = (String) keySerializer.Deserialize(reader);
            TValue value = (TValue) valueSerializer.Deserialize(reader);

            reader.MoveToContent();

            Add(key, value);
        }
    }

    public System.Xml.Schema.XmlSchema GetSchema()
    {
        return null;
    }

    public override String ToString()
    {
        String result = "";

        foreach (String key in this.Dictionary.Keys)
        {
            if (result!="") 
              result +=  Environment.NewLine;
            result = String.Format("{0}{1}={2}", result, key, this[key]);
        }

        return result;
    }    
}


class TestApp
{    
    public static void Main()
    { 
        DictionaryCollection<Int32> dictionary = new DictionaryCollection<Int32>();
        XmlSerializer serializer = new XmlSerializer(typeof(DictionaryCollection<Int32>));        
        
        dictionary.Add("key1", 10);
        dictionary.Add("key2", 20);
        dictionary["key3"] = 30;

        using (StreamWriter writer = new StreamWriter("sample.xml"))
        {
            XmlTextWriter xmlWriter = new XmlTextWriter(writer);
            xmlWriter.Formatting = Formatting.Indented;
            serializer.Serialize(xmlWriter, dictionary);
        }
        
        StreamReader reader = new StreamReader("sample.xml");
        XmlTextReader xmlReader = new XmlTextReader(reader);
        dictionary = (DictionaryCollection<Int32>) serializer.Deserialize(xmlReader);
        reader.Close();

        Console.WriteLine(dictionary);        
    }
}