using System;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters;
using System.Runtime.Serialization.Formatters.Binary;
using System.Reflection;

[Serializable]
public class Person : ISerializable
{           
    public String name;

    public Person()
    {
    }

    public void GetObjectData(SerializationInfo info, StreamingContext context)
    {
        Type thisType = this.GetType();
        MemberInfo[] serializableMembers = 
            FormatterServices.GetSerializableMembers(thisType, context);
        foreach (MemberInfo serializableMember in serializableMembers)
        {            
            //      NonSerializedAttribute
            if (!(Attribute.IsDefined(serializableMember, 
                        typeof(NonSerializedAttribute))))
            {                    
                info.AddValue(serializableMember.Name, 
                        ((FieldInfo)serializableMember).GetValue(this));
            }
        }
    }

    protected Person(SerializationInfo info, 
                        StreamingContext context)
    {
        Type thisType = this.GetType();
        MemberInfo[] serializableMembers = 
                FormatterServices.GetSerializableMembers(thisType, context);
        foreach (MemberInfo serializableMember in serializableMembers) 
        {
            FieldInfo fieldInformation = (FieldInfo)serializableMember;
            if (!(Attribute.IsDefined(serializableMember, 
                                        typeof(NonSerializedAttribute))))
            {                    
                fieldInformation.SetValue(this, 
                            info.GetValue(fieldInformation.Name, 
                            fieldInformation.FieldType));
            }
        }
    }
} // Person

public class App
{
    public static void Main()
    {
        using (Stream stream = new MemoryStream())
        {
            IFormatter formatter = new BinaryFormatter();
            Person person = new Person();
            person.name = "";
            Console.WriteLine(": {0}", person.name);
            formatter.Serialize(stream, person);

            stream.Position = 0;
            Person personRestored = (Person) formatter.Deserialize(stream);
            Console.WriteLine(": {0}", personRestored.name);
       }
    }
}
