using System;
using System.Threading;
using System.Messaging;

using Seva.Msmq;
using Seva.ApplicationLogs; 
using Seva.Programm;

using Seva.Math.DistributedIntegration;

namespace MsmqApplication
{
    class MessageReceiver
    {
        private MsmqServer<IntegrationRequest, IntegrationAnswer> msmq;
            
        private Integration integration;
            
        public MessageReceiver(String queueReceiveName)
        {
            msmq = new MsmqServer<IntegrationRequest, IntegrationAnswer>(queueReceiveName, 
                QueueFormatter.Xml);         
            msmq.ProcessMessage += OnReceive;
            
            integration = new Integration();
        }   
        
        public void BeginReceive()
        {
            msmq.BeginReceive();            
        }   
        
        public IntegrationAnswer OnReceive(Object sender, IntegrationRequest request, 
            MessageQueue queueResponse)
        {       
            DateTime startTime = DateTime.Now;          
            IntegrationAnswer answer = integration.Run(request); 
            TimeSpan duration = DateTime.Now - startTime;
            Log.Message("Request: {0},answer to {1} in {2}", 
                request.ToString(), queueResponse.QueueName, duration);
            return answer;
        }   
    }       
    
    class ServerProgram
    {
        static public void Main()       
        {       
        Log.Message(Messages.msmqConfig2,  ProgrammConfig.OrderQueueName);
            
        Log.Start(Messages.msmqCheckQueues);
        MessageReceiver recevier = new MessageReceiver(ProgrammConfig.OrderQueueName);
        Log.Finish();
            
        Log.Message(Messages.msmqAwaiting);
        recevier.BeginReceive();    
        Thread.Sleep(-1);      
        }
    }
}