using System; 
using System.Collections;
using System.Collections.Generic;

namespace Seva.Data
{
    public class Cache<Key, Value>
        where Key: IComparable where Value: MarshalByRefObject
    {
        private List<Key> keys;
        private List<Int32> hits;
        private List<Value> values; 
        
        private Int32 maxCount;
            
        public Cache(Int32 aMaxCount)
        {
            maxCount = aMaxCount;
            keys = new List<Key>();
            hits = new List<Int32>();
            values = new List<Value>();         
        }
        
        public Value Find(Key key)
        {
            int n=  keys.IndexOf(key);
            if (n==-1) 
                return null;
            else
            {
                hits[n]++;
                return values[n];
            }   
        }           
        
        public Value Add(Key key, Value value)
        {
            CheckSize();
            hits.Add(0);
            keys.Add(key);
            values.Add(value);
            
            return value;
        }   

        private void CheckSize()
        {
            if (keys.Count<maxCount) return;
            
            int n=-1, min=0;
            
            for (Int32 i=0; i<keys.Count; i++)
                if (n==-1 || hits[i]<min)
                {
                    n =  i;
                    min = hits[i];
                }               

            if (n==-1) return;

            hits.RemoveAt(n);   
            keys.RemoveAt(n);   
            values.RemoveAt(n); 
        }           
    }
}