using System;
using System.IO;
using System.Collections.Generic;
using System.EnterpriseServices;
using System.EnterpriseServices.CompensatingResourceManager;

[assembly: ApplicationActivation(ActivationOption.Server)]
[assembly: ApplicationAccessControl(false)]
[assembly: ApplicationCrmEnabled]
[assembly: Description("COM+ CRM sample")]

 public static class ESLog
 {
     static string logName = "com_log.txt";
     
     public static void Message(String message, params Object[] objects)
     {
         Message(String.Format(message, objects));
     }
     
     public static void Message(String message) 
     {
         FileStream file = new FileStream(logName, 
                 FileMode.Append, FileAccess.Write);
         using (StreamWriter writer = new StreamWriter(file)) 
         {         
             writer.WriteLine(message);
         }
     }
 }        


public static class StreamLog
{      
    public static void Save(LogRecord log)
    {        
        if (log.Record is object[])
        {    
            object[] record = (object[])log.Record;
            string fileName = (string) record[0];
            byte[] buffer = (byte[]) record[1];                         
       
            using (FileStream file = new FileStream(fileName, 
                FileMode.Create, FileAccess.Write)) 
            {         
                file.Write(buffer, 0, buffer.Length);
            }
        }
    } // Save() 
} // StreamLog
    
public class StreamCash
{       
    private string fileName;
        
    private MemoryStream stream;        
    
    public  MemoryStream Stream
    {
        get { return stream;}
    }         
    
    public  string FileName
    {
        get { return fileName;}
    }      
            
    public StreamCash(string streamFileName)
    {    
        fileName = streamFileName;
        stream = new MemoryStream();
    }
    
    public void Reopen() 
    {
        if (!stream.CanRead) 
        {
            stream.Close();
            byte[] buffer = stream.ToArray();
            stream.Dispose();
            stream = new MemoryStream();
            stream.Write(buffer, 0, buffer.Length);
            stream.Seek(0, SeekOrigin.Begin);
        }                
    }            
           
    public void Read() 
    {         
        byte[] buffer = new byte[32*1024];
        using (Stream inputStream = new FileStream(fileName, FileMode.Open, FileAccess.Read))
        {            
            while (true)
            {
                int read = inputStream.Read(buffer, 0, buffer.Length);
                if (read <= 0) 
                {
                    break;
                }    
                Stream.Write(buffer, 0, read);
            }  
        }    
    } // Read()                     
    
} // StreamLog       

public class StreamCrm: ServicedComponent//: MarshalByRefObject // ServicedComponent
{
    private Dictionary<string, StreamCash> cash;
        
    public StreamCrm()
    {
        cash = new Dictionary<string, StreamCash>();
    }
    
    private StreamCash CheckCash(string fileName, bool read)
    {
        StreamCash streamCash;
        string key = Path.GetFullPath(fileName).ToLower();
        if (!cash.ContainsKey(key))
        {
            ESLog.Message("crm: {0} - new file, keys: {1}", key, 
                cash.Keys.Count);
            streamCash = new StreamCash(fileName);
            cash.Add(key, streamCash);                                 
            if (read)
            {
                streamCash.Read();
            };   
        }
        else
        {
            ESLog.Message("crm: {0} - old file", key);
            streamCash = cash[key];
            streamCash.Reopen();
        };            
        
        return streamCash;    
    }    
    
    public MemoryStream ReadFromFile(string fileName)
    {
        return CheckCash(fileName, true).Stream;
        //return new StreamReader(streamLog.Stream);          
    }
    
    public MemoryStream WriteToFile(String fileName)
    {
        return CheckCash(fileName, false).Stream;                     
        
    }  
           
    public object[] Flush(String fileName)
    {
        StreamCash streamCash = CheckCash(fileName, false);
        streamCash.Stream.Close();        
        object[] record = new object[2];
        record[0] = fileName;
        record[1] = streamCash.Stream.ToArray();
                
        return record;//StreamLog(fileName, streamCash.Stream);
    }   
    
    public static Clerk CreateClerk()
    {
        return new Clerk(typeof(StreamCompensator), "Compensator",  
                            CompensatorOptions.AllPhases);
    }    
}

public class StreamCompensator : Compensator
{        
    private bool prepared = false;

    public override void BeginPrepare ()
    {
    }
    
    public override bool PrepareRecord(LogRecord log)
    {
        prepared = false;
        ESLog.Message("prepare...");
        if (!(log.Record is object[]))
        {
            ESLog.Message(".. not object[]!");
            return true;
        }
        object[] record = log.Record as object[];
        if (record.Length != 2)
        {
            ESLog.Message(".. length<>2!");
            return true;
        }
        if (!(record[0] is string) || !(record[1] is byte[]))
        {
            ESLog.Message(".. [0] || [1]!");
            return true;
        }
        ESLog.Message("...ok");
        prepared = true;
        return false;
    }
    
    public override bool EndPrepare ()
    {
        ESLog.Message("Prepared: {0}", prepared);
        return prepared;
    }

    public override void BeginCommit (bool commit)
    {
    }

    public override bool CommitRecord (LogRecord log)
    {
        StreamLog.Save(log);                
                    
        return true;
    }
    
    public override void EndCommit ()
    {
    }
    
    public override void BeginAbort (bool abort)
    {
    }
    
    public override bool AbortRecord (LogRecord log)
    {
        return true;
    }
    
    public override void EndAbort ()
    {
    }    
} // StreamCrm


[Transaction(TransactionOption.Required)]
public class SampleCrmClient2: ServicedComponent
{
    public SampleCrmClient2()
    {
    }

    public string DoSomeWork2(StreamCrm crm, string fileName)
    {
        ESLog.Message("client2: {0}", fileName);
        using (StreamReader reader = new StreamReader(crm.ReadFromFile(fileName)))
        {            
            string line = reader.ReadLine();
            ESLog.Message("client2: {0} read", line);
            return line;
        } 
     }
}

[Transaction(TransactionOption.RequiresNew)]
public class SampleCrmClient1: ServicedComponent
{
    public SampleCrmClient1()
    {
    }
    
    public void DoSomeWork1(string dir)
    {      
        const string fileName1 = "sample1.txt";   
        const string fileName2 = "sample2.txt";

        Environment.CurrentDirectory = dir;

        ESLog.Message(dir);
        
        StreamCrm crm = new StreamCrm();
                             
        Clerk clerk =  StreamCrm.CreateClerk();
        
        using (StreamWriter writer = new StreamWriter(crm.WriteToFile(fileName1)))
        {            
            writer.Write(String.Format("Current directory : [{0}]", 
                Environment.CurrentDirectory));
        }         

        clerk.WriteLogRecord(crm.Flush(fileName1));
        
        String tempString = "";
        
        using (SampleCrmClient2 client2 = new SampleCrmClient2())
        {            
            tempString = client2.DoSomeWork2(crm, Path.GetFullPath(fileName1));
            ESLog.Message("client1: {0} read", tempString);
        } 
        
        using (StreamWriter writer = new StreamWriter(crm.WriteToFile(fileName2)))
        {            
            writer.WriteLine(String.Format("Read from [{0}]:", fileName1));
            writer.WriteLine(tempString);
        }         
                       
        
        clerk.WriteLogRecord(crm.Flush(fileName2));          
                        
        ContextUtil.SetComplete();        
    }            
}     

class CrmTest
{    
    public static void Main()
    {        
        SampleCrmClient1 client1 = new SampleCrmClient1();
        try
        {
            client1.DoSomeWork1(Environment.CurrentDirectory);          
        }
        finally
        {
            client1.Dispose();
        }
    }
}

