using System;
using System.IO;
using System.Collections.Generic;
using System.EnterpriseServices;
using System.EnterpriseServices.CompensatingResourceManager;

namespace Seva.StreamCrm
{
    public static class StreamLog
    {      
        public static void Save(LogRecord log)
        {        
            if (log.Record is object[])
            {    
                object[] record = (object[])log.Record;
                string fileName = (string) record[0];
                byte[] buffer = (byte[]) record[1];                         
           
                using (FileStream file = new FileStream(fileName, 
                    FileMode.Create, FileAccess.Write)) 
                {         
                    file.Write(buffer, 0, buffer.Length);
                }
            }
        } // Save() 
    } // StreamLog
        
    public class StreamCash
    {       
        private string fileName;
            
        private MemoryStream stream;        
        
        public  MemoryStream Stream
        {
            get { return stream;}
        }         
        
        public  string FileName
        {
            get { return fileName;}
        }      
                
        public StreamCash(string streamFileName)
        {    
            fileName = streamFileName;
            stream = new MemoryStream();
        }
        
        public void Reopen() 
        {
            if (!stream.CanRead) 
            {
                stream.Close();
                byte[] buffer = stream.ToArray();
                stream.Dispose();
                stream = new MemoryStream();
                stream.Write(buffer, 0, buffer.Length);
                stream.Seek(0, SeekOrigin.Begin);
            }                
        }            
               
        public void Read() 
        {         
            byte[] buffer = new byte[32*1024];
            using (Stream inputStream = new FileStream(fileName, FileMode.Open, FileAccess.Read))
            {            
                while (true)
                {
                    int read = inputStream.Read(buffer, 0, buffer.Length);
                    if (read <= 0) 
                    {
                        break;
                    }    
                    Stream.Write(buffer, 0, read);
                }  
            }    
        } // Read()                     
        
    } // StreamLog       

    public class StreamCrm//: ServicedComponent
    {
        private Dictionary<string, StreamCash> cash;
            
        public StreamCrm()
        {
            cash = new Dictionary<string, StreamCash>();
        }
        
        private StreamCash CheckCash(string fileName, bool read)
        {
            StreamCash streamCash;
            string key = Path.GetFullPath(fileName).ToLower();
            if (!cash.ContainsKey(key))
            {                
                streamCash = new StreamCash(fileName);
                cash.Add(key, streamCash);                                 
                if (read)
                {
                    streamCash.Read();
                };   
            }
            else
            {
                streamCash = cash[key];
                streamCash.Reopen();
            };            
            
            return streamCash;    
        }    
        
        public MemoryStream ReadFromFile(string fileName)
        {
            return CheckCash(fileName, true).Stream;
            //return new StreamReader(streamLog.Stream);          
        }
        
        public MemoryStream WriteToFile(String fileName)
        {
            return CheckCash(fileName, false).Stream;                     
            
        }  
               
        public object[] Flush(String fileName)
        {
            StreamCash streamCash = CheckCash(fileName, false);
            streamCash.Stream.Close();        
            object[] record = new object[2];
            record[0] = fileName;
            record[1] = streamCash.Stream.ToArray();
                    
            return record;//StreamLog(fileName, streamCash.Stream);
        }   
        
        public static Clerk CreateClerk()
        {
            return new Clerk(typeof(StreamCompensator), "Compensator",  
                                CompensatorOptions.AllPhases);
        }    
    }

    public class StreamCompensator : Compensator
    {        
        private bool prepared = false;

        public override void BeginPrepare ()
        {
        }
        
        public override bool PrepareRecord(LogRecord log)
        {
            prepared = false;
            
            if (!(log.Record is object[]))
            {                
                return true;
            }
            object[] record = log.Record as object[];
            if (record.Length != 2)
            {
                return true;
            }
            if (!(record[0] is string) || !(record[1] is byte[]))
            {
                return true;
            }
            prepared = true;
            return false;
        }
        
        public override bool EndPrepare ()
        {
            return prepared;
        }

        public override void BeginCommit (bool commit)
        {
        }

        public override bool CommitRecord (LogRecord log)
        {
            StreamLog.Save(log);                
                        
            return true;
        }
        
        public override void EndCommit ()
        {
        }
        
        public override void BeginAbort (bool abort)
        {
        }
        
        public override bool AbortRecord (LogRecord log)
        {
            return true;
        }
        
        public override void EndAbort ()
        {
        }    
    } // StreamCrm

} // SevaStreamCrm    
