using System;
using System.IO;
using System.Reflection;
using System.Xml;
using System.Xml.Serialization;
using System.Collections.Generic;

using Microsoft.Web.Services3;
using Microsoft.Web.Services3.Design;
using Microsoft.Web.Services3.Security;

[assembly:AssemblyVersionAttribute("1.0.0.0")]

namespace Seva.WS.Assertions
{   
    //
    public class TimeAssertion: PolicyAssertion
    {
        private TimeIntervals intervals;
        public TimeAssertion(): base()
        {
            intervals = null;
        }
        
        public override void ReadXml(XmlReader reader, 
            IDictionary<string, Type> extensions)
        {                        
            string fileName = reader.GetAttribute("file");
            if (fileName == null)
                throw new Exception("Attribute 'file' not found in policy");
            
            //   -       
            reader.Read();
                     
            using (StreamReader timeReader = new StreamReader(fileName))
            {
                XmlSerializer serializer = new XmlSerializer(typeof(TimeIntervals));
                intervals = (TimeIntervals) serializer.Deserialize(new XmlTextReader(timeReader));
            }
        }

        public override SoapFilter CreateClientInputFilter(FilterCreationContext context)
        {            
            return null;
        }

        public override SoapFilter CreateClientOutputFilter(FilterCreationContext context)
        {
            return null;
        }

        public override SoapFilter CreateServiceInputFilter(FilterCreationContext context)
        {
            return new CheckTimeFilter(intervals, context);
        }

        public override SoapFilter CreateServiceOutputFilter(FilterCreationContext context)
        {
            return null;
        }
        
    }
    
    public class CheckTimeFilter : SoapFilter
    {
        private TimeIntervals intervals;
            
        public CheckTimeFilter(TimeIntervals intervals, 
            FilterCreationContext context)
        {
            this.intervals = intervals;
        }
            
        public override SoapFilterResult ProcessMessage(SoapEnvelope envelope)
        {
            if (! intervals.Check(DateTime.Now))
                throw new Exception("Service is unavailable now.");
            return SoapFilterResult.Continue;
        }
    }
    
    [Serializable]
    public class TimeInterval
    {
        [XmlAttribute("From")]        
        public string FromField
        {
            set {From = TimeSpan.Parse(value);}
            get {return From.ToString();}
        }
        
        [XmlAttribute("To")]
        public string ToField
        {
            set {To = TimeSpan.Parse(value);}
            get {return To.ToString();}
        }
        
        [XmlAttribute("Inverse")]
        public bool Inverse;
        
        [XmlIgnore]
        public TimeSpan From;
            
        [XmlIgnore]
        public TimeSpan To;
            
        public override string ToString()
        {
            string result = String.Format("{0} - {1}", From, To);
            if (Inverse)
                result = "(not in) " + result;
            return result;
        }
        
        public bool Check(DateTime Time)
        {
            bool result = (From <= Time.TimeOfDay) && (Time.TimeOfDay <= To);
            if (Inverse)
                result = ! result;
            
            return result;
        }
    }

    [XmlRoot("TimeIntervals")]    
    public class TimeIntervals: List<TimeInterval>
    {
        public TimeIntervals()
        {
        }
            
        public bool Check(DateTime Time)
        {
            foreach (TimeInterval interval in this)
                if (interval.Check(Time))
                     return true;
            return false;
        }
    }
        
    //~ public class TimeIntervals
    //~ {
        //~ private List<TimeInterval> intervalsField;
            
        //~ [XmlArray("TimeIntervals")]
        //~ public List<TimeInterval> Intervals 
        //~ {
            //~ get {return intervalsField;}
        //~ }
        
        //~ public TimeIntervals()
        //~ {
            //~ intervalsField = new List<TimeInterval>();
        //~ }
            
        //~ public bool Check(DateTime Time)
        //~ {
            //~ foreach (TimeInterval interval in Intervals)
                //~ if (interval.Check(Time))
                     //~ return true;
            //~ return false;
        //~ }
    //~ }
}