//  SevaRemotingMsmq.cs
using System;
using System.IO;
using System.Collections;

using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Messaging;
using System.Messaging;

using System.Text.RegularExpressions;

//      MSMQ   
//    
using Seva.Msmq;

//      
//   Remoting / MSMQ
namespace Seva.Remoting.MsmqChannel
{   
    //     
    public class MessageProperties 
    {   
        public const string Uri            = "__Uri";  
        public const string ObjectUri      = "__ObjectUri";                
    }

    //    MsmqSender  MsmqReceiver
    public class MsmqBase: MarshalByRefObject, IChannel
    {        
        //    IChannel 
        public int ChannelPriority { get { return 1;} }
        public string ChannelName { get{ return "msmq";} }              
        
        //  URL      
        public string Parse(string url, out string objectUri)
        {            
            return Utils.ParseUrl(url, out objectUri);            
        }    
      }
    
    
    //    
    public class MsmqChannelSender: MsmqBase, IChannelSender
    {
                                
        public MsmqChannelSender(IDictionary properties, 
            IClientChannelSinkProvider clientSinkProvider)
        {                        
            //         MSMQ,
            //                 
            if (clientSinkProvider!=null)
            {
                throw new NotSupportedException(
                    "  ."); 
            }            
        }
                
        //    IChannelSender,
        //          
        public IMessageSink CreateMessageSink(string url, 
            object channelData, out string objectUri)
        {      
            //   URL    
            string remoteobjectUrl = Utils.ParseUrl(url, ChannelName, 
                out objectUri);
            if (remoteobjectUrl == null) 
                return null;       
            
            //  
            return new MsmqClientChannelSink(remoteobjectUrl);                    
        }   
    } // Seva.Remoting.MsmqChannel.Msmq.MsmqSender
    
    
    //    
    public class MsmqClientChannelSink: IMessageSink
    {
        //  MSMQ,     IMessage         
        //      ,   
        //   void   
        private MsmqClient<IMessage, IMessage> msmqClient;
            
        //    MSMQ     (endpoint)
        private string queuePath = null;
        private string objectUri = null;
            
        //   
        public MsmqClientChannelSink(string remoteObjectUri) 
        {
            //   URL       
            queuePath = Utils.ParseUrl(remoteObjectUri, out objectUri);
            //objectUri = remoteObjectUri;
            
            //   MSMQ,     
            //     
            msmqClient = new MsmqClient<IMessage, IMessage>(queuePath, null, 
                QueueFormatter.Binary);   
        }        

        public IMessage SyncProcessMessage(IMessage msg)
        {
           throw new NotSupportedException(
                "    OneWayAttribute."); 
        }
       
        //            
        public IMessageCtrl AsyncProcessMessage(IMessage message, 
            IMessageSink replySink)
        {
            //    c   
            message.Properties[MessageProperties.ObjectUri] = objectUri;
                           
            //    MSMQ    
            msmqClient.Send(message);            
            
            return null;    
        }    

        //   -     
        public IMessageSink NextSink
        { 
            get { return null;  }
        }                        
    }  // Seva.Remoting.MsmqChannel.MsmqClientChannelSink
    
    //   
    public class MsmqChannelReceiver: MsmqBase, IChannelReceiver
    {
        // - MSMQ    Remoting
        private MsmqServer<IMessage, IMessage> msmqServer;
            
        //   
        private IServerChannelSink sink;
            
        //     
        private ServerChannelSinkStack stack;           
         
        //     
        public object ChannelData 
        {
            get { return null; }
        }
            
        public MsmqChannelReceiver(IDictionary properties,
            IServerChannelSinkProvider serverSinkProvider)
        {
            //   MSMQ
            msmqServer = new MsmqServer<IMessage, IMessage>(
                properties["queue"].ToString(), QueueFormatter.Binary);
            msmqServer.ProcessMessage += OnReceive;
            
            if (serverSinkProvider!=null)
            {
                throw new NotSupportedException(
                    "  ."); 
            }
            //       
            //  MsmqReceiver,   -  MsmqServer,
            //       
            sink = ChannelServices.CreateServerChannelSinkChain(null, this);
            
            //                                       
            stack = new ServerChannelSinkStack();
            stack.Push(sink, null);
            
            //     
            StartListening(null);
        }            
            
        //   IChannelReceiver,  
        // URL   URI
        public virtual string[] GetUrlsForUri(string objectURI)
        {
            return new string[] {objectURI};
        }
                        
        //    MSMQ
        private IMessage OnReceive(Object sender, IMessage request, 
            MessageQueue queueResponse)
        {                    
            return ProcessMessage(request);
        }       
        
        // ,   
        public void StartListening(Object data)
        {
            msmqServer.BeginReceive();
        }
                
        public void StopListening(Object data)
        {
            msmqServer.EndReceive();
        }      
                        
        //     Remoting
        private IMessage ProcessMessage(IMessage request)
        {
            IMessage response = null;
            
            object uri = request.Properties[MessageProperties.ObjectUri];
            
            if (uri == null) return null;                
         
            string url = uri.ToString();   
            
            //     __URI
            request.Properties[MessageProperties.Uri] = uri;  
            
            //     
            Stream responseStream = null;
            ITransportHeaders responseHeaders = null;

            sink.ProcessMessage(stack, request, null, null,  
                out response, out responseHeaders, out responseStream);           

            return response;
        }
    } // Seva.Remoting.MsmqChannel.MsmqReceiver
        
    //     
    abstract class Utils    
    {
        //    URL    
        public static string ParseUrl(string url, string channelName, 
            out string objectUri)
        {
            objectUri = null;
            
            Regex re = new Regex(@"^" + channelName + @"\:\/\/(.+)$");                 
            Match m = re.Match(url); 
            if (!m.Success) return null;
                
            string sinkUri =  m.Groups[1].Value;
            Utils.ParseUrl(url, out objectUri);
            
            return sinkUri;
        }
        
        //    URL    
        //    MSMQ
        public static string ParseUrl(string url, out string objectUri)
        {
            string queuePath = null;
            objectUri = null;
            
            Regex r = new Regex(@"^(.*)(\\|\/)(.+)$"); 
            Match m = r.Match(url); 
            if (m.Success)
            {
                objectUri = m.Groups[3].Value;
                queuePath = m.Groups[1].Value;
            }
            else
            {
                throw new Exception("  URI   " + url);
            }
        
           // Console.WriteLine("{0} -> {1} + {2}", url, queuePath, objectUri);    
            return queuePath;
        }

    } // Seva.Remoting.MsmqChannel.Utils
} // Seva.Remoting.MsmqChannel
//  SevaRemotingMsmq.cs
